@@ -17,7 +17,7 @@
 #endif /* HAVE_CONFIG_H */
 
 #ifdef AUTH_MYSQL
-static const char rcsid[] = "$Id: auth_mysql.c,v 1.34 2001/10/27 14:41:10 chris Exp $";
+static const char rcsid[] = "$Id: auth_mysql.c,v 1.46 2002/06/25 20:28:00 chris Exp $";
 
 #include <sys/types.h> /* BSD needs this here, apparently. */
 
@@ -36,6 +36,7 @@ static const char rcsid[] = "$Id: auth_mysql.c,v 1.34 2001/10/27 14:41:10 chris
 
 #include "auth_mysql.h"
 #include "authswitch.h"
+#include "config.h"
 #include "md5.h"
 #include "stringmap.h"
 #include "util.h"
@@ -62,7 +63,13 @@ char *apop_query_template =
        "AND popbox.domain_name = '$(domain)' "
        "AND popbox.domain_name = domain.domain_name";
 
-static char *substitute_query_params(const char *temp, const char *local_part, const char *domain);
+char *onlogin_query_template = NULL;
+
+/* GID used to access mail spool (if any). */
+int use_gid;
+gid_t mail_gid;
+
+static char *substitute_query_params(const char *temp, const char *user, const char *local_part, const char *domain, const char *clienthost, const char *serverhost);
 
 /* MD5 crypt(3) routines. This is here so that you can migrate passwords from
  * the modern /etc/shadow crypt_md5 format used (optionally) by recent
@@ -86,8 +93,7 @@ static unsigned char itoa64[] = /* 0 ... 63 => ascii - 64 */
         "./0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz";
 
 /* to64:
- * Convert a string into a different base.
- */
+ * Convert a string into a different base. */
 static void to64(char *s, unsigned long v, int n)
 {
     while (--n >= 0) {
@@ -97,8 +103,7 @@ static void to64(char *s, unsigned long v, int n)
 }
 
 /* crypt_md5:
- * Poul-Henning Kamp's crypt(3)-alike using MD5.
- */
+ * Poul-Henning Kamp's crypt(3)-alike using MD5. */
 static char *crypt_md5(const char *pw, const char *salt)
 {
     const char *magic = "$1$";
@@ -223,7 +228,7 @@ static char *crypt_md5(const char *pw, const char *salt)
  * This file is public domain and comes with NO WARRANTY of any kind
  */
 
-/* hash_password:
+/* mysql_hash_password:
  * MySQL password-hashing routine. */
 static void mysql_hash_password(unsigned long *result, const char *password) {
     register unsigned long nr=1345345333L, add=7, nr2=0x12345671L;
@@ -241,7 +246,7 @@ static void mysql_hash_password(unsigned long *result, const char *password) {
     return;
 }
 
-/* make_scrambled_password:
+/* mysql_make_scrambled_password:
  * MySQL function to form a password hash and turn it into a string. */
 static void mysql_make_scrambled_password(char *to, const char *password) {
     unsigned long hash_res[2];
@@ -253,8 +258,7 @@ static void mysql_make_scrambled_password(char *to, const char *password) {
 
 
 /* strclr:
- * Clear a string.
- */
+ * Clear a string. */
 static void strclr(char *s) {
     memset(s, 0, strlen(s));
 }
@@ -262,48 +266,72 @@ static void strclr(char *s) {
 /* auth_mysql_init:
  * Initialise the database connection driver. Clears the config directives
  * associated with the database so that a user cannot recover them with a
- * debugger.
- */
-extern stringmap config; /* in main.c */
+ * debugger. */
 MYSQL *mysql;
 
 int auth_mysql_init() {
-    char *username = NULL, *password = NULL, *hostname = NULL, *database = NULL, *localhost = "localhost";
-    item *I;
+    char *username = NULL, *password = NULL, *hostname = NULL, *database = NULL, *localhost = "localhost", *s;
     int ret = 0;
 
-    if ((I = stringmap_find(config, "auth-mysql-username"))) username = (char*)I->v;
+    if ((s = config_get_string("auth-mysql-username")))
+        username = s;
     else {
-        print_log(LOG_ERR, _("auth_mysql_init: no auth-mysql-username directive in config"));
+        log_print(LOG_ERR, _("auth_mysql_init: no auth-mysql-username directive in config"));
         goto fail;
     }
 
-    if ((I = stringmap_find(config, "auth-mysql-password"))) password = (char*)I->v;
+    if ((s = config_get_string("auth-mysql-password")))
+        password = s;
     else {
-        print_log(LOG_WARNING, _("auth_mysql_init: no auth-mysql-password directive in config; using blank password"));
+        log_print(LOG_WARNING, _("auth_mysql_init: no auth-mysql-password directive in config; using blank password"));
         password = "";
     }
 
-    if ((I = stringmap_find(config, "auth-mysql-database"))) database = (char*)I->v;
+    if ((s = config_get_string("auth-mysql-database")))
+        database = s;
     else {
-        print_log(LOG_ERR, _("auth_mysql_init: no auth-mysql-database directive in config"));
+        log_print(LOG_ERR, _("auth_mysql_init: no auth-mysql-database directive in config"));
         goto fail;
     }
 
-    if ((I = stringmap_find(config, "auth-mysql-hostname"))) hostname = (char*)I->v;
+    if ((s = config_get_string("auth-mysql-hostname")))
+        hostname = s;
     else hostname = localhost;
 
-    if ((I = stringmap_find(config, "auth-mysql-pass-query"))) user_pass_query_template = (char*)I->v;
-    if ((I = stringmap_find(config, "auth-mysql-apop-query"))) apop_query_template = (char*)I->v;
+    /* Obtain query templates. The special string `none' means `don't use
+     * any query for this action'. */
+    if ((s = config_get_string("auth-mysql-pass-query")))
+        user_pass_query_template = s;
+    if (strcmp(user_pass_query_template, "none") == 0)
+        user_pass_query_template = NULL;
+    
+    if ((s = config_get_string("auth-mysql-apop-query")))
+        apop_query_template = s;
+    if (strcmp(apop_query_template, "none") == 0)
+        apop_query_template = NULL;
+
+    /* This is an optional action to put a row into the database after a
+     * successful login, for POP-before-SMTP relaying. */
+    if ((s = config_get_string("auth-mysql-onlogin-query")))
+        onlogin_query_template = s;
+
+    /* Obtain gid to use */
+    if ((s = config_get_string("auth-mysql-mail-group"))) {
+        if (!parse_gid(s, &mail_gid)) {
+            log_print(LOG_ERR, _("auth_mysql_init: auth-mysql-mail-group directive `%s' does not make sense"), s);
+            goto fail;
+        }
+        use_gid = 1;
+    }
 
     mysql = mysql_init(NULL);
     if (!mysql) {
-        print_log(LOG_ERR, _("auth_mysql_init: mysql_init: failed"));
+        log_print(LOG_ERR, _("auth_mysql_init: mysql_init: failed"));
         goto fail;
     }
 
     if (mysql_real_connect(mysql, hostname, username, password, database, 0, NULL, 0) != mysql) {
-        print_log(LOG_ERR, "auth_mysql_init: mysql_real_connect: %s", mysql_error(mysql));
+        log_print(LOG_ERR, "auth_mysql_init: mysql_real_connect: %s", mysql_error(mysql));
         mysql_close(mysql);
         mysql = NULL;
         goto fail;
@@ -324,60 +352,40 @@ extern int verbose; /* in main.c */
 
 /* auth_mysql_new_apop:
  * Attempt to authenticate a user via APOP, using the template SELECT query in
- * the config file or the default defined above otherwise.
- */
-authcontext auth_mysql_new_apop(const char *name, const char *timestamp, const unsigned char *digest, const char *host /* unused */) {
+ * the config file or the default defined above otherwise. */
+authcontext auth_mysql_new_apop(const char *name, const char *local_part, const char *domain, const char *timestamp, const unsigned char *digest, const char *clienthost /* unused */, const char *serverhost) {
     char *query = NULL;
     authcontext a = NULL;
-    char *local_part = NULL;
-    const char *domain;
-    item *I;
-    int use_gid = 0;
-    gid_t gid;
+    char *who;
 
-    if (!mysql) return NULL;
+    who = username_string(name, local_part, domain);
 
-    /* Obtain gid to use */
-    if ((I = stringmap_find(config, "auth-mysql-mail-group"))) {
-        if (!parse_gid((char*)I->v, &gid)) {
-            print_log(LOG_ERR, _("auth_mysql_new_apop: auth-mysql-mail-group directive `%s' does not make sense"), (char*)I->v);
-            return NULL;
-        }
-        use_gid = 1;
-    }
+    if (!mysql || !apop_query_template) return NULL;
 
-    domain = name + strcspn(name, "@%!");
-    if (domain == name || !*domain) return NULL;
-    ++domain;
-    
-    local_part = xmalloc(domain - name);
-    if (!local_part) return NULL;
-    memset(local_part, 0, domain - name);
-    strncpy(local_part, name, domain - name - 1);
-    
     if (mysql_ping(mysql) == -1) {
-        print_log(LOG_ERR, "auth_mysql_new_apop: mysql_ping: %s", mysql_error(mysql));
+        log_print(LOG_ERR, "auth_mysql_new_apop: mysql_ping: %s", mysql_error(mysql));
         return NULL;
     }
 
     /* Obtain the actual query to use. */
-    query = substitute_query_params(apop_query_template, local_part, domain);
-    if (!query) goto fail;
+    if (!(query = substitute_query_params(apop_query_template, name, local_part, domain, NULL, serverhost)))
+        goto fail;
 
     if (verbose)
-        print_log(LOG_DEBUG, "auth_mysql_new_apop: SQL query: %s", query);
+        log_print(LOG_DEBUG, "auth_mysql_new_apop: SQL query: %s", query);
 
     if (mysql_query(mysql, query) == 0) {
-        MYSQL_RES *result = mysql_store_result(mysql);
+        MYSQL_RES *result;
         int i;
 
+        result = mysql_store_result(mysql);
         if (!result) {
-            print_log(LOG_ERR, "auth_mysql_new_apop: mysql_store_result: %s", mysql_error(mysql));
+            log_print(LOG_ERR, "auth_mysql_new_apop: mysql_store_result: %s", mysql_error(mysql));
             goto fail;
         }
 
         if (mysql_field_count(mysql) != 4) {
-            print_log(LOG_ERR, "auth_mysql_new_apop: %d fields returned by query, should be 4: mailbox location, password hash, unix user, mailbox type", mysql_field_count(mysql));
+            log_print(LOG_ERR, "auth_mysql_new_apop: %d fields returned by query, should be 4: mailbox location, password hash, unix user, mailbox type", mysql_field_count(mysql));
             goto fail;
         }
 
@@ -385,19 +393,20 @@ authcontext auth_mysql_new_apop(const char *name, const char *timestamp, const u
         case 0:
             break;
         case 1: {
-                MYSQL_ROW row = mysql_fetch_row(result);
+                MYSQL_ROW row;
                 unsigned long *lengths;
                 struct passwd *pw;
                 unsigned char this_digest[16];
                 MD5_CTX ctx;
                 uid_t uid;
-
+                
+                row = mysql_fetch_row(result);
                 /* These are "can't happen" errors */
                 if (!row || !(lengths = mysql_fetch_lengths(result))) break;
 
                 /* Verify that this user has a plaintext password. */
                 if (strncmp(row[1], "{plaintext}", 11) != 0) {
-                    print_log(LOG_WARNING, _("auth_mysql_new_apop: attempted APOP login by %s@%s, who does not have a plaintext password"), local_part, domain);
+                    log_print(LOG_WARNING, _("auth_mysql_new_apop: attempted APOP login by %s, who does not have a plaintext password"), who);
                     break;
                 }
                 
@@ -409,45 +418,40 @@ authcontext auth_mysql_new_apop(const char *name, const char *timestamp, const u
 
                 /* User was lying */
                 if (memcmp(this_digest, digest, 16)) {
-                    print_log(LOG_WARNING, _("auth_mysql_new_apop: failed login for %s@%s"), local_part, domain);
+                    log_print(LOG_WARNING, _("auth_mysql_new_apop: failed login for %s"), who);
                     break;
                 }
 
                 /* User was not lying (about her password) */
                 if (!parse_uid((const char*)row[2], &uid)) {
-                    print_log(LOG_ERR, _("auth_mysql_new_apop: unix user `%s' for %s@%s does not make sense"), row[3], local_part, domain);
+                    log_print(LOG_ERR, _("auth_mysql_new_apop: unix user `%s' for %s does not make sense"), row[3], who);
                     break;
                 }
 
                 pw = getpwuid(uid);
 
                 if (!pw) {
-                    print_log(LOG_ERR, "auth_mysql_new_apop: getpwuid(%d): %m", (int)uid);
+                    log_print(LOG_ERR, "auth_mysql_new_apop: getpwuid(%d): %m", (int)uid);
                     break;
                 }
 
-                /* We will let the program figure out which sort of mailbox
-                 * to use later.
-                 */
-                a = authcontext_new(pw->pw_uid, use_gid ? gid : pw->pw_gid,
-                                    row[3], row[0], pw->pw_dir, domain);
+                a = authcontext_new(pw->pw_uid, use_gid ? mail_gid : pw->pw_gid, row[3], row[0], pw->pw_dir);
 
                 break;
             }
 
         default:
-            print_log(LOG_ERR, _("auth_mysql_new_apop: database inconsistency: query for %s returned %d rows"), name, i);
+            log_print(LOG_ERR, _("auth_mysql_new_apop: database inconsistency: query for %s returned %d rows"), name, i);
             break;
         }
 
         mysql_free_result(result);
         
     } else {
-        print_log(LOG_ERR, "auth_mysql_new_apop: mysql_query: %s", mysql_error(mysql));
+        log_print(LOG_ERR, "auth_mysql_new_apop: mysql_query: %s", mysql_error(mysql));
     }
 
 fail:
-    if (local_part) xfree(local_part);
     if (query) xfree(query);
 
     return a;
@@ -456,60 +460,38 @@ fail:
 /* auth_mysql_new_user_pass:
  * Attempt to authenticate a user via USER/PASS, using the template SELECT
  * query in the config file or the default defined above otherwise. */
-authcontext auth_mysql_new_user_pass(const char *user, const char *pass, const char *host /* unused */) {
-    char *query = NULL;
+authcontext auth_mysql_new_user_pass(const char *user, const char *local_part, const char *domain, const char *pass, const char *clienthost /* unused */, const char *serverhost) {
+    char *query = NULL, *who;
     authcontext a = NULL;
-    char *local_part = NULL;
-    const char *domain;
-    char *p;
-    unsigned char *q;
-    item *I;
-    int use_gid = 0;
-    gid_t gid;
 
-    if (!mysql) return NULL;
-
-    /* Obtain gid to use */
-    if ((I = stringmap_find(config, "auth-mysql-mail-group"))) {
-        if (!parse_gid((char*)I->v, &gid)) {
-            print_log(LOG_ERR, _("auth_mysql_new_apop: auth-mysql-mail-group directive `%s' does not make sense"), (char*)I->v);
-            return NULL;
-        }
-        use_gid = 1;
-    }
-
-    domain = user + strcspn(user, "@%!");
-    if (domain == user || !*domain) return NULL;
-    ++domain;
-    
-    local_part = xmalloc(domain - user);
-    if (!local_part) return NULL;
-    memset(local_part, 0, domain - user);
-    strncpy(local_part, user, domain - user - 1);
+    who = username_string(user, local_part, domain);
     
+    if (!mysql || !user_pass_query_template) return NULL;
+
     if (mysql_ping(mysql) == -1) {
-        print_log(LOG_ERR, "auth_mysql_new_user_pass: mysql_ping: %s", mysql_error(mysql));
+        log_print(LOG_ERR, "auth_mysql_new_user_pass: mysql_ping: %s", mysql_error(mysql));
         return NULL;
     }
 
     /* Obtain the actual query to use. */
-    query = substitute_query_params(user_pass_query_template, local_part, domain);
-    if (!query) goto fail;
+    if (!(query = substitute_query_params(user_pass_query_template, user, local_part, domain, NULL, serverhost)))
+        goto fail;
 
     if (verbose)
-        print_log(LOG_DEBUG, "auth_mysql_new_user_pass: SQL query: %s", query);
+        log_print(LOG_DEBUG, "auth_mysql_new_user_pass: SQL query: %s", query);
 
     if (mysql_query(mysql, query) == 0) {
-        MYSQL_RES *result = mysql_store_result(mysql);
+        MYSQL_RES *result;
         int i;
 
+        result = mysql_store_result(mysql);
         if (!result) {
-            print_log(LOG_ERR, "auth_mysql_new_user_pass: mysql_store_result: %s", mysql_error(mysql));
+            log_print(LOG_ERR, "auth_mysql_new_user_pass: mysql_store_result: %s", mysql_error(mysql));
             goto fail;
         }
 
         if (mysql_field_count(mysql) != 4) {
-            print_log(LOG_ERR, "auth_mysql_new_user_pass: %d fields returned by query, should be 4: mailbox location, password hash, unix user, mailbox type", mysql_field_count(mysql));
+            log_print(LOG_ERR, "auth_mysql_new_user_pass: %d fields returned by query, should be 4: mailbox location, password hash, unix user, mailbox type", mysql_field_count(mysql));
             goto fail;
         }
 
@@ -517,12 +499,14 @@ authcontext auth_mysql_new_user_pass(const char *user, const char *pass, const c
         case 0:
             break;
         case 1: {
-                MYSQL_ROW row = mysql_fetch_row(result);
+                MYSQL_ROW row;
                 unsigned long *lengths;
                 char *pwhash;
                 struct passwd *pw;
                 int authok = 0;
                 uid_t uid;
+                
+                row = mysql_fetch_row(result);
 
                 /* These are "can't happen" errors */
                 if (!row || !(lengths = mysql_fetch_lengths(result))) break;
@@ -556,97 +540,158 @@ authcontext auth_mysql_new_user_pass(const char *user, const char *pass, const c
                             break;
 
                         default:
-                            print_log(LOG_ERR, _("auth_mysql_new_user_pass: %s@%s has mysql password type, but hash is of incorrect length %d"), local_part, domain, n);
+                            log_print(LOG_ERR, _("auth_mysql_new_user_pass: %s has password type mysql, but hash is of incorrect length %d"), who, n);
                             break;
                     }
-                } else if (strncmp(pwhash, "{md5}", 4) == 0 || *pwhash != '{') {
-                    /* Straight MD5 password. */
-                    MD5_CTX ctx;
-                    unsigned char digest[16];
-                    char hexhash[33] = {0};
-                    MD5Init(&ctx);
-                    MD5Update(&ctx, (unsigned char*)pass, strlen(pass));
-                    MD5Final(digest, &ctx);
+                } else if (strncmp(pwhash, "{md5}", 5) == 0 || *pwhash != '{') {
+                    /* Straight MD5 password. But this might be either in hex
+                     * or base64 encoding. */
+                    if (*pwhash == '{')
+                        pwhash += 5;
 
-                    for (p = hexhash, q = digest; q < digest + 16; ++q, p += 2) snprintf(p, 3, "%02x", (unsigned)*q);
-
-                    if (strcasecmp(hexhash, pwhash + 5) == 0 || strcasecmp(hexhash, pwhash) == 0) authok = 1;
+                    if (strlen(pwhash) == 32) {
+                        /* Hex. */
+                        if (strcasecmp(pwhash, md5_digest_str(pass, strlen(pass), 0)) == 0)
+                            authok = 1;
+                    } else if (strlen(pwhash) == 24) {
+                        /* Base 64. */
+                        if (strcmp(pwhash, md5_digest_str(pass, strlen(pass), 1)) == 0)
+                            authok = 1;
+                    } else
+                        /* Doesn't make sense. */
+                        log_print(LOG_ERR, _("auth_mysql_new_user_pass: %s has password type md5, but hash is of incorrect length"), who);
                 } else {
                     /* Unknown format. */
-                    print_log(LOG_ERR, _("auth_mysql_new_user_pass: %s@%s has unknown password format `%.*s'"), local_part, domain, 2 + strcspn(pwhash + 1, "}"), pwhash);
+                    log_print(LOG_ERR, _("auth_mysql_new_user_pass: %s has unknown password format `%.*s'"), who, 2 + strcspn(pwhash + 1, "}"), pwhash);
                     break;
                 }
 
                 if (!authok) {
-                    print_log(LOG_ERR, _("auth_mysql_new_user_pass: %s@%s failed login with wrong password"), local_part, domain);
+                    log_print(LOG_ERR, _("auth_mysql_new_user_pass: %s failed login with wrong password"), who);
                     break;
                 }
 
                 if (!parse_uid((const char*)row[2], &uid)) {
-                    print_log(LOG_ERR, _("auth_mysql_new_user_pass: unix user `%s' for %s@%s does not make sense"), row[3], local_part, domain);
+                    log_print(LOG_ERR, _("auth_mysql_new_user_pass: unix user `%s' for %s does not make sense"), row[3], who);
                     break;
                 }
 
                 pw = getpwuid(uid);
 
                 if (!pw) {
-                    print_log(LOG_ERR, "auth_mysql_new_user_pass: getpwuid(%d): %m", (int)uid);
+                    log_print(LOG_ERR, "auth_mysql_new_user_pass: getpwuid(%d): %m", (int)uid);
                     break;
                 }
 
-                a = authcontext_new(pw->pw_uid, use_gid ? gid : pw->pw_gid,
-                                    row[3], row[0], pw->pw_dir, domain);
+                a = authcontext_new(pw->pw_uid, use_gid ? mail_gid : pw->pw_gid, row[3], row[0], pw->pw_dir);
                 break;
             }
 
         default:
-            print_log(LOG_ERR, _("auth_mysql_new_user_pass: database inconsistency: query for %s@%s returned %d rows"), local_part, domain, i);
+            log_print(LOG_ERR, _("auth_mysql_new_user_pass: database inconsistency: query for %s returned %d rows"), who, i);
             break;
         }
 
         mysql_free_result(result);
-    } else {
-        print_log(LOG_ERR, "auth_mysql_new_user_pass: mysql_query: %s", mysql_error(mysql));
-    }
+    } else
+        log_print(LOG_ERR, "auth_mysql_new_user_pass: mysql_query: %s", mysql_error(mysql));
 
 fail:
-    if (local_part) xfree(local_part);
-    if (query) xfree(query);
+    xfree(query);
 
     return a;
 }
 
+/* auth_mysql_onlogin:
+ * If specified, perform a query (action) after a successful login. The
+ * variables substituted in the template are $(local_part), $(domain) and
+ * $(clienthost), the username, domain, and connecting client host. */
+void auth_mysql_onlogin(const authcontext A, const char *clienthost, const char *serverhost) {
+    char *query;
+
+    if (!mysql || !onlogin_query_template) return;
+
+    if (mysql_ping(mysql) == -1) {
+        log_print(LOG_ERR, "auth_mysql_onlogin: mysql_ping: %s", mysql_error(mysql));
+        return;
+    }
+
+    query = substitute_query_params(onlogin_query_template, A->user, A->local_part, A->domain, clienthost, serverhost);
+    if (!query)
+        return;
+
+    if (verbose)
+        log_print(LOG_DEBUG, _("auth_mysql_onlogin: SQL query: %s"), query);
+
+    if (mysql_query(mysql, query) == 0) {
+        MYSQL_RES *result;
+        /* It's possible that the user put a query in which returned some rows.
+         * This is bogus but there's not a lot we can do; to avoid leaking
+         * memory or confusing the database, we obtain and free a result, and
+         * log a warning. */
+        result = mysql_store_result(mysql);
+        if (result) {
+            log_print(LOG_WARNING, _("auth_mysql_onlogin: supplied SQL query returned %d rows, which is dubious"), mysql_num_rows(result));
+            mysql_free_result(result);
+        }
+    } else
+        log_print(LOG_ERR, "auth_mysql_onlogin: mysql_query: %s", mysql_error(mysql));
+
+    xfree(query);
+}
+
+/* auth_mysql_postfork:
+ * Post-fork cleanup. */
+void auth_mysql_postfork() {
+    mysql = NULL; /* XXX */
+}
+
 /* auth_mysql_close:
- * Close the database connection.
- */
+ * Close the database connection. */
 void auth_mysql_close() {
     if (mysql) mysql_close(mysql);
 }
 
 /* substitute_query_params
  * Given a query template, a localpart and a domain, return a copy of the
- * template with the fields filled in.
- */
-static char *substitute_query_params(const char *template, const char *local_part, const char *domain) {
-    char *query, *l, *d;
+ * template with the fields filled in. */
+static char *substitute_query_params(const char *template, const char *user, const char *local_part, const char *domain, const char *clienthost, const char *serverhost) {
+    char *query, *u = NULL, *l = NULL, *d = NULL, *c = NULL, *s = NULL;
     struct sverr err;
 
     /* Form escaped copies of the user and domain. */
-    if (!(l = xmalloc(strlen(local_part) * 2 + 1)))
-	return NULL;
-    mysql_escape_string(l, local_part, strlen(local_part));
+    u = xmalloc(strlen(user) * 2 + 1);
+    mysql_escape_string(u, user, strlen(user));
+    
+    if (local_part) {
+        l = xmalloc(strlen(local_part) * 2 + 1);
+        mysql_escape_string(l, local_part, strlen(local_part));
+    }
 
-    if (!(d = xmalloc(strlen(domain) * 2 + 1))) {
-	xfree(l);
-	return NULL;
+    if (domain) {
+        d = xmalloc(strlen(domain) * 2 + 1);
+        mysql_escape_string(d, domain, strlen(domain));
+    }
+
+    if (clienthost) {
+        c = xmalloc(strlen(clienthost) * 2 + 1);
+        mysql_escape_string(c, clienthost, strlen(clienthost));
+    }
+
+    if (serverhost) {
+        s = xmalloc(strlen(serverhost) * 2 + 1);
+        mysql_escape_string(s, serverhost, strlen(serverhost));
     }
-    mysql_escape_string(d, domain, strlen(domain));
 
     /* Do the substitution. */
-    query = substitute_variables(template, &err, 2, "local_part", l, "domain", d);
-    if (!query)
-        print_log(LOG_ERR, _("substitute_query_params: %s near `%.16s'"), err.msg, template + err.offset);
-    
+    query = substitute_variables(template, &err, 5, "user", u, "local_part", l, "domain", d, "clienthost", c, "serverhost", s);
+
+    if (!query && err.code != sv_nullvalue)
+        log_print(LOG_ERR, _("substitute_query_params: %s near `%.16s'"), err.msg, template + err.offset);
+
+    xfree(s);
+    xfree(c);
+    xfree(u);
     xfree(l);
     xfree(d);
     return query;
