@@ -15,6 +15,7 @@
  *  pass        client's password from PASS command
  *  digest      client's digest sent with APOP command, in hex
  *  clienthost  client's IP address/hostname
+ *  serverhost  server's IP address/hostname
  *
  * The program should respond with a similarly formatted string containing the
  * following variables:
@@ -44,7 +45,7 @@
 #endif /* HAVE_CONFIG_H */
 
 #ifdef AUTH_OTHER
-static const char rcsid[] = "$Id: auth_other.c,v 1.23 2002/03/18 23:47:33 chris Exp $";
+static const char rcsid[] = "$Id: auth_other.c,v 1.24 2002/06/25 20:28:00 chris Exp $";
 
 #include <sys/types.h>
 
@@ -471,7 +472,7 @@ fail:
 
 /* auth_other_new_apop:
  * Attempt to authenticate a user using APOP, via the child program. */
-authcontext auth_other_new_apop(const char *name, const char *local_part, const char *domain, const char *timestamp, const unsigned char *digest, const char *host) {
+authcontext auth_other_new_apop(const char *name, const char *local_part, const char *domain, const char *timestamp, const unsigned char *digest, const char *clienthost, const char *serverhost) {
 #define MISSING(k)     do { log_print(LOG_ERR, _("auth_other_new_apop: missing key `%s' in response"), (k)); goto fail; } while(0)
 #define INVALID(k, v)  do { log_print(LOG_ERR, _("auth_other_new_apop: invalid value `%s' for key `%s' in response"), (v), (k)); goto fail; } while(0)
     char digeststr[33];
@@ -486,9 +487,9 @@ authcontext auth_other_new_apop(const char *name, const char *local_part, const
     for (p = digeststr, q = digest; q < digest + 16; p += 2, ++q)
         sprintf(p, "%02x", (unsigned int)*q);
     if (local_part && domain) {
-        if (!auth_other_send_request(7, "method", "APOP", "user", name, "local_part", local_part, "domain", domain, "timestamp", timestamp, "digest", digeststr, "clienthost", host))
+        if (!auth_other_send_request(8, "method", "APOP", "user", name, "local_part", local_part, "domain", domain, "timestamp", timestamp, "digest", digeststr, "clienthost", clienthost, "serverhost", serverhost))
             return NULL;
-    } else if (!auth_other_send_request(5, "method", "APOP", "user", name, "timestamp", timestamp, "digest", digeststr, "clienthost", host))
+    } else if (!auth_other_send_request(6, "method", "APOP", "user", name, "timestamp", timestamp, "digest", digeststr, "clienthost", clienthost, "serverhost", serverhost))
         return NULL;
 
     if (!(S = auth_other_recv_response()))
@@ -538,7 +539,7 @@ fail:
 
 /* auth_other_new_user_pass:
  * Attempt to authenticate a user using USER/PASS, via the child program. */
-authcontext auth_other_new_user_pass(const char *user, const char *local_part, const char *domain, const char *pass, const char *host) {
+authcontext auth_other_new_user_pass(const char *user, const char *local_part, const char *domain, const char *pass, const char *clienthost, const char *serverhost) {
 #define MISSING(k)     do { log_print(LOG_ERR, _("auth_other_new_user_pass: missing key `%s' in response"), (k)); goto fail; } while(0)
 #define INVALID(k, v)  do { log_print(LOG_ERR, _("auth_other_new_user_pass: invalid value `%s' for key `%s' in response"), (v), (k)); goto fail; } while(0)
     stringmap S;
@@ -548,9 +549,9 @@ authcontext auth_other_new_user_pass(const char *user, const char *local_part, c
     if (!authchild_pid) auth_other_start_child();
 
     if (local_part && domain) {
-        if (!auth_other_send_request(6, "method", "PASS", "user", user, "local_part", local_part, "domain", domain, "pass", pass, "clienthost", host))
+        if (!auth_other_send_request(7, "method", "PASS", "user", user, "local_part", local_part, "domain", domain, "pass", pass, "clienthost", clienthost, "serverhost", serverhost))
             return NULL;
-    } else if (!auth_other_send_request(4, "method", "PASS", "user", user, "pass", pass, "clienthost", host))
+    } else if (!auth_other_send_request(5, "method", "PASS", "user", user, "pass", pass, "clienthost", clienthost, "serverhost", serverhost))
         return NULL;
 
     if (!(S = auth_other_recv_response()))
@@ -600,13 +601,13 @@ fail:
 
 /* auth_other_onlogin:
  * Pass details of a successful login to the authentication program. */
-void auth_other_onlogin(const authcontext A, const char *host) {
+void auth_other_onlogin(const authcontext A, const char *clienthost, const char *serverhost) {
     stringmap S;
     item *I;
 
     if (!authchild_pid) auth_other_start_child();
 
-    if (!auth_other_send_request(5, "method", "ONLOGIN", "user", A->user, "local_part", A->local_part, "domain", A->domain, "clienthost", host)
+    if (!auth_other_send_request(6, "method", "ONLOGIN", "user", A->user, "local_part", A->local_part, "domain", A->domain, "clienthost", clienthost, "serverhost", serverhost)
         || !(S = auth_other_recv_response()))
         return;
     
