@@ -11,7 +11,7 @@
 #endif /* HAVE_CONFIG_H */
 
 #ifdef AUTH_PAM
-static const char rcsid[] = "$Id: auth_pam.c,v 1.18 2001/10/14 01:13:37 chris Exp $";
+static const char rcsid[] = "$Id: auth_pam.c,v 1.25 2002/06/25 20:28:00 chris Exp $";
 
 #include <sys/types.h> /* BSD needs this here, it seems. */
 
@@ -26,15 +26,12 @@ static const char rcsid[] = "$Id: auth_pam.c,v 1.18 2001/10/14 01:13:37 chris Ex
 
 #include "auth_pam.h"
 #include "authswitch.h"
-#include "stringmap.h"
+#include "config.h"
 #include "util.h"
 
-char *mem;
-
 /* auth_pam_conversation:
  * PAM conversation function, used to transmit the password supplied by the
- * user to the PAM modules for authentication.
- */
+ * user to the PAM modules for authentication. */
 int auth_pam_conversation(int num_msg, const struct pam_message **msg, struct pam_response **resp, void *appdata_ptr) {
     const struct pam_message **m;
     struct pam_response *r;
@@ -47,7 +44,7 @@ int auth_pam_conversation(int num_msg, const struct pam_message **msg, struct pa
     /* Assume that any prompt is asking for a password */
     for (m = msg, r = *resp; m < msg + num_msg; ++m, ++r) {
         if ((*m)->msg_style == PAM_PROMPT_ECHO_OFF) {
-            r->resp = mem = strdup((char*)appdata_ptr);
+            r->resp = xstrdup((char*)appdata_ptr);
             r->resp_retcode = 0;
         }
     }
@@ -56,42 +53,36 @@ int auth_pam_conversation(int num_msg, const struct pam_message **msg, struct pa
 }
 
 /* auth_pam_new_user_pass:
- * Attempt to authenticate user and pass using PAM.
- */
-extern stringmap config;
-
-authcontext auth_pam_new_user_pass(const char *user, const char *pass, const char *host /* unused */) {
+ * Attempt to authenticate user and pass using PAM. This is not a
+ * virtual-domains authenticator, so it only looks at user. */
+authcontext auth_pam_new_user_pass(const char *user, const char *local_part, const char *domain, const char *pass, const char *clienthost /* unused */, const char *serverhost) {
     pam_handle_t *pamh = NULL;
     struct passwd pw, *pw2;
     int r, n = PAM_SUCCESS;
     authcontext a = NULL;
     struct pam_conv conv;
     char *facility;
-    item *I;
+    char *s;
     int use_gid = 0;
     gid_t gid = 99;
-    const char *x;
 
     /* Check the this isn't a virtual-domain user. */
-    x = user + strcspn(user, "@%!");
-    if (*x) return NULL;
+    if (local_part) return NULL;
 
     /* Copy the password structure, since it is in static storage and may
-     * get overwritten by calls in the PAM code.
-     */
+     * get overwritten by calls in the PAM code. */
     pw2 = getpwnam(user);
     if (!pw2) return NULL;
     else memcpy(&pw, pw2, sizeof(pw));
 
     /* Obtain facility name. */
-    I = stringmap_find(config, "auth-pam-facility");
-    if (I) facility = (char*)I->v;
-    else facility = AUTH_PAM_FACILITY;
+    if (!(facility = config_get_string("auth-pam-facility")))
+        facility = AUTH_PAM_FACILITY;
 
     /* Obtain gid to use */
-    if ((I = stringmap_find(config, "auth-pam-mail-group"))) {
-        if (!parse_gid((char*)I->v, &gid)) {
-            print_log(LOG_ERR, _("auth_pam_new_user_pass: auth-pam-mail-group directive `%s' does not make sense"), (char*)I->v);
+    if ((s = config_get_string("auth-pam-mail-group"))) {
+        if (!parse_gid(s, &gid)) {
+            log_print(LOG_ERR, _("auth_pam_new_user_pass: auth-pam-mail-group directive `%s' does not make sense"), s);
             return NULL;
         }
         use_gid = 1;
@@ -104,9 +95,19 @@ authcontext auth_pam_new_user_pass(const char *user, const char *pass, const cha
     r = pam_start(facility, user, &conv, &pamh);
 
     if (r != PAM_SUCCESS) {
-        print_log(LOG_ERR, "auth_pam_new_user_pass: pam_start: %s", pam_strerror(pamh, r));
+        log_print(LOG_ERR, "auth_pam_new_user_pass: pam_start: %s", pam_strerror(pamh, r));
         return NULL;
     }
+    
+    /* We want to be able to test against the client IP; make the remote host
+     * information available to the PAM stack. */
+    r = pam_set_item(pamh, PAM_RHOST, clienthost);
+    
+    if (r != PAM_SUCCESS) {
+        log_print(LOG_ERR, "auth_pam_new_user_pass: pam_start: %s", pam_strerror(pamh, r));
+        return NULL;
+    }
+
 
     /* Authenticate user. */
     r = pam_authenticate(pamh, 0);
@@ -116,13 +117,13 @@ authcontext auth_pam_new_user_pass(const char *user, const char *pass, const cha
         r = pam_acct_mgmt(pamh, PAM_SILENT);
         if (r == PAM_SUCCESS) {
             /* Succeeded; figure out the mailbox name later. */
-            a = authcontext_new(pw.pw_uid, use_gid ? gid : pw.pw_gid, NULL, NULL, pw2->pw_dir, NULL);
-        } else print_log(LOG_ERR, "auth_pam_new_user_pass: pam_acct_mgmt(%s): %s", user, pam_strerror(pamh, r));
-    } else print_log(LOG_ERR, "auth_pam_new_user_pass: pam_authenticate(%s): %s", user, pam_strerror(pamh, r));
+            a = authcontext_new(pw.pw_uid, use_gid ? gid : pw.pw_gid, NULL, NULL, pw2->pw_dir);
+        } else log_print(LOG_ERR, "auth_pam_new_user_pass: pam_acct_mgmt(%s): %s", user, pam_strerror(pamh, r));
+    } else log_print(LOG_ERR, "auth_pam_new_user_pass: pam_authenticate(%s): %s", user, pam_strerror(pamh, r));
 
     r = pam_end(pamh, n);
 
-    if (r != PAM_SUCCESS) print_log(LOG_ERR, "auth_pam_new_user_pass: pam_end: %s", pam_strerror(pamh, r));
+    if (r != PAM_SUCCESS) log_print(LOG_ERR, "auth_pam_new_user_pass: pam_end: %s", pam_strerror(pamh, r));
 
     return a;
 }
