@@ -9,7 +9,7 @@
  *
  */
 
-static const char rcsid[] = "$Id: cfgdirectives.c,v 1.13 2002/03/18 23:47:33 chris Exp $";
+static const char rcsid[] = "$Id: cfgdirectives.c,v 1.16 2002/06/08 19:41:38 chris Exp $";
 
 #ifdef HAVE_CONFIG_H
 #include "configuration.h"
@@ -26,7 +26,10 @@ char *cfgdirectives[] = {
     "strip-domain",
     "timeout-seconds",
     "log-facility",
+    "log-stderr",
+    "apop-only",
     "mailbox",
+    "no-detach",
 
 #ifdef AUTH_PAM
     /* auth-pam options */
@@ -89,6 +92,13 @@ char *cfgdirectives[] = {
     "auth-perl-pass",
     "auth-perl-onlogin",
 #endif /* AUTH_PERL */
+
+#ifdef AUTH_FLATFILE
+    "auth-flatfile-enable",
+    "auth-flatfile-mail-user",
+    "auth-flatfile-mail-group",
+    "auth-flatfile-passwd-file",
+#endif /* AUTH_FLATFILE */
  
 #if defined(MBOX_BSD) && defined(MBOX_BSD_SAVE_INDICES)
     "mailspool-index",
@@ -97,6 +107,10 @@ char *cfgdirectives[] = {
 #ifdef USE_TCP_WRAPPERS
     "tcp-wrappers-name",
 #endif
+
+#ifdef USE_DRAC
+    "drac-server",
+#endif
     
     /* final entry must be NULL */
     NULL};
