@@ -6,7 +6,7 @@
  *
  */
 
-static const char rcsid[] = "$Id: connection.c,v 1.31 2002/03/18 23:47:33 chris Exp $";
+static const char rcsid[] = "$Id: connection.c,v 1.34 2002/06/25 20:28:00 chris Exp $";
 
 #ifdef HAVE_CONFIG_H
 #include "configuration.h"
@@ -69,8 +69,7 @@ static char *make_timestamp(const char *domain) {
     if (n != sizeof(buffer)) {
         /* OK, we need to get some pseudo-random data from rand(3).
          * FIXME This is bad from a security PoV, and should be replaced by
-         * hashing some rapidly-changing data.
-         */
+         * hashing some rapidly-changing data. */
         unsigned char *p;
         for (p = buffer; p < buffer + sizeof(buffer); ++p)
             *p = (unsigned char)(rand() & 0xff);
@@ -90,18 +89,31 @@ static char *make_timestamp(const char *domain) {
 /* connection_new:
  * Create a connection object from a socket. */
 connection connection_new(int s, const struct sockaddr_in *sin, const char *domain) {
-    connection c = 0;
+    int n;
+    connection c = NULL;
 
     c = xcalloc(1, sizeof *c);
 
     c->s = s;
     memcpy(&(c->sin), sin, sizeof(struct sockaddr_in));
 
+    n = sizeof(c->sin_local);
+    if (getsockname(s, (struct sockaddr *)&(c->sin_local), &n) < 0) {
+      log_print(LOG_WARNING, _("connection_new: getsockname error %d"), errno);
+      goto fail;
+    }
+
+    c->remote_ip = xstrdup(inet_ntoa(c->sin.sin_addr));
+    c->local_ip = xstrdup(inet_ntoa(c->sin_local.sin_addr));
+
     if (domain) c->domain = xstrdup(domain);
 
-    c->idstr = xmalloc(strlen(inet_ntoa(sin->sin_addr)) + 1 + (domain ? strlen(domain) : 0) + 16);
-    if (domain) sprintf(c->idstr, "[%d]%s/%s", s, inet_ntoa(sin->sin_addr), domain);
-    else sprintf(c->idstr, "[%d]%s", s, inet_ntoa(sin->sin_addr));
+    if (!c->domain)
+      c->domain = xstrdup(c->local_ip);
+
+    c->idstr = xmalloc(strlen(c->remote_ip) + 1 + (c->domain ? strlen(c->domain) : 0) + 16);
+    if (c->domain) sprintf(c->idstr, "[%d]%s/%s", s, c->remote_ip, c->domain);
+    else sprintf(c->idstr, "[%d]%s", s, c->remote_ip);
 
     c->p = c->buffer = xmalloc(c->bufferlen = MAX_POP3_LINE);
     if (!c->buffer) goto fail;
@@ -139,6 +151,8 @@ void connection_delete(connection c) {
     if (c->m) (c->m)->delete(c->m);
 
     if (c->domain)    xfree(c->domain);
+    if (c->remote_ip) xfree(c->remote_ip);
+    if (c->local_ip)  xfree(c->local_ip);
     if (c->idstr)     xfree(c->idstr);
     if (c->buffer)    xfree(c->buffer);
     if (c->timestamp) xfree(c->timestamp);
@@ -161,7 +175,10 @@ ssize_t connection_read(connection c) {
     do {
         n = read(c->s, c->p, c->buffer + c->bufferlen - c->p);
     } while (n == -1 && errno == EINTR);
-    if (n > 0) c->p += n;
+    if (n > 0) {
+        c->p += n;
+        c->nrd += n;    /* Keep track of data transferred. */
+    }
     return n;
 }
 
@@ -248,9 +265,31 @@ pop3command connection_parsecommand(connection c) {
  * Create a new pop3command object. */
 pop3command pop3command_new(const char *s) {
     pop3command p;
+    const char *q;
     int i;
     
-    p = xcalloc(1, sizeof *p);
+    p = xcalloc(sizeof *p, 1);
+
+    /* Ugly. PASS is a special case, because we permit a password to contain
+     * spaces. */
+    q = s + strspn(s, " \t");
+    if (strncasecmp(q, "PASS ", 5) == 0) {
+        /* Manual parsing. */
+        p->cmd = PASS;
+        p->toks = xcalloc(sizeof *p->toks, 1);
+        
+        p->toks->str = xstrdup(q);
+        chomp(p->toks->str);
+        p->toks->str[4] = 0;
+        
+        p->toks->toks = xcalloc(sizeof(char*), 2);
+        p->toks->toks[0] = p->toks->str;
+        p->toks->toks[1] = p->toks->str + 5;
+        
+        p->toks->num = 2;
+
+        return p;
+    }
 
     p->cmd = UNKNOWN;
     p->toks = tokens_new(s, " \t");
@@ -290,6 +329,8 @@ int connection_sendresponse(connection c, const int success, const char *s) {
     xfree(x);
     if (verbose)
         log_print(LOG_DEBUG, _("connection_sendresponse: client %s: sent `%s %s'"), c->idstr, success? "+OK" : "-ERR", s);
+    if (m > 0)
+        c->nwr += m;
     return (m == l);
 }
 
@@ -304,6 +345,8 @@ int connection_sendline(connection c, const char *s) {
     snprintf(x, l, "%s\r\n", s);
     m = xwrite(c->s, x, l = strlen(x));
     xfree(x);
+    if (m > 0)
+        c->nwr += m;
     return (m == l);
 }
 
