@@ -4,7 +4,7 @@
  *
  * Copyright (c) 2001 Chris Lightfoot. All rights reserved.
  *
- * $Id: connection.h,v 1.11 2001/04/23 23:45:40 chris Exp $
+ * $Id: connection.h,v 1.13 2002/06/25 20:28:00 chris Exp $
  *
  */
 
@@ -31,7 +31,11 @@ enum pop3_state {authorisation, transaction, update};
 typedef struct _connection {
     int s;                  /* connected socket                 */
     struct sockaddr_in sin; /* name of peer                     */
+    char *remote_ip;        /* ASCII remote IP address          */
+    struct sockaddr_in sin_local; /* name of local side         */
+    char *local_ip;         /* ASCII local IP address           */
     char *idstr;            /* some identifying information     */
+    size_t nrd, nwr;        /* number of bytes read/written     */
     
     char *domain;           /* associated domain suffix         */
     char *buffer;           /* buffer from peer                 */
