@@ -7,7 +7,7 @@
  */
 
 static const char copyright[] = "$Copyright: (c) 2001 Chris Lightfoot. All rights reserved. $";
-static const char rcsid[] = "$Id: main.c,v 1.59 2001/10/23 15:07:44 chris Exp $";
+static const char rcsid[] = "$Id: main.c,v 1.70 2002/06/25 20:28:00 chris Exp $";
 
 #ifdef HAVE_CONFIG_H
 #include "configuration.h"
@@ -49,17 +49,17 @@ int deny_severity  = LOG_NOTICE;
 
 /* The socket send buffer is set to this, so that we don't end up in a
  * position that we send so much data that the client will not have received
- * all of it before we time them out.
- */
+ * all of it before we time them out. */
 #define MAX_DATA_IN_FLIGHT      8192
 
 /* Data structure representing the config file, and global variable which we
- * set from it.
- */
+ * set from it. */
 stringmap config;
 
 /* Various configuration options. */
 extern int append_domain;           /* Do we automatically try user@domain if user alone fails to authenticate? In pop3.c. */
+extern int strip_domain;            /* Do we automatically try user if user@domain fails to authenticate? */
+extern int apop_only;               /* Quit after receiving USER. */
 int log_stderr;                     /* Are log messages also sent to standard error? */
 int verbose;                        /* Should we be verbose about data going to/from the client? */
 int timeout_seconds = 30;           /* How long a period of inactivity may elapse before a client is dropped. */
@@ -69,7 +69,7 @@ volatile int num_running_children = 0;  /* How many children are active. */
 
 char *pidfile = NULL;               /* The name of a PID file to use; if NULL, don't use one. */
 
-char *tcpwrappersname = "tpop3d";   /* The daemon name to give to TCP Wrappers. */
+char *tcpwrappersname;              /* The daemon name to give to TCP Wrappers. */
 
 /* Variables representing the state of the server. */
 int post_fork = 0;                  /* Is this a child handling a connection. */
@@ -84,13 +84,11 @@ size_t max_connections;             /* Number of connection slots allocated. */
  * connections_pre_select, then calls select, then calls listeners_ and
  * connections_post_select. In the event that a server is forked to handle a
  * client, fork_child is called. The global variables listeners and
- * connections are used to handle this procedure.
- */
+ * connections are used to handle this procedure. */
 
 /* find_free_connection:
- * Find a free connection slot.
- */
-connection *find_free_connection() {
+ * Find a free connection slot. */
+connection *find_free_connection(void) {
     connection *J;
     for (J = connections; J < connections + max_connections; ++J)
         if (!*J) return J;
@@ -98,8 +96,7 @@ connection *find_free_connection() {
 }
 
 /* remove_connection:
- * Remove a connection from the list.
- */
+ * Remove a connection from the list. */
 void remove_connection(connection c) {
     connection *J;
     for (J = connections; J < connections + max_connections; ++J)
@@ -107,8 +104,7 @@ void remove_connection(connection c) {
 }
 
 /* listeners_pre_select:
- * Called before the main select(2) so listening sockets can be polled.
- */
+ * Called before the main select(2) so listening sockets can be polled. */
 void listeners_pre_select(int *n, fd_set *readfds, fd_set *writefds, fd_set *exceptfds) {
     item *t;
     vector_iterate(listeners, t) {
@@ -120,8 +116,7 @@ void listeners_pre_select(int *n, fd_set *readfds, fd_set *writefds, fd_set *exc
 
 /* listeners_post_select:
  * Called after the main select(2) to allow listening sockets to sort
- * themselves out.
- */
+ * themselves out. */
 void listeners_post_select(fd_set *readfds, fd_set *writefds, fd_set *exceptfds) {
     item *t;
     vector_iterate(listeners, t) {
@@ -129,26 +124,28 @@ void listeners_post_select(fd_set *readfds, fd_set *writefds, fd_set *exceptfds)
         if (FD_ISSET(L->s, readfds)) {
             struct sockaddr_in sin;
             size_t l = sizeof(sin);
-            int s = accept(L->s, (struct sockaddr*)&sin, (int *)&l);
-            int a = MAX_DATA_IN_FLIGHT;
+            int s, a = MAX_DATA_IN_FLIGHT;
 
+            /* XXX socklen_t mess... */
+            s = accept(L->s, (struct sockaddr*)&sin, (int *)&l);
+            
             if (s == -1) {
-                if (errno != EAGAIN) print_log(LOG_ERR, "net_loop: accept: %m");
+                if (errno != EAGAIN) log_print(LOG_ERR, "net_loop: accept: %m");
             }
 #ifdef USE_TCP_WRAPPERS
             else if (!hosts_ctl(tcpwrappersname, STRING_UNKNOWN, inet_ntoa(sin.sin_addr), STRING_UNKNOWN)) {
-                print_log(LOG_ERR, "net_loop: tcp_wrappers: connection from %s refused", inet_ntoa(sin.sin_addr));
+                log_print(LOG_ERR, "net_loop: tcp_wrappers: connection from %s refused", inet_ntoa(sin.sin_addr));
                 close(s);
             }
 #endif
             else if (setsockopt(s, SOL_SOCKET, SO_SNDBUF, &a, sizeof(a)) == -1) {
                 /* Set a small send buffer so that we get usefully blocking
                  * writes. */
-                print_log(LOG_ERR, "listeners_post_select: setsockopt: %m");
+                log_print(LOG_ERR, "listeners_post_select: setsockopt: %m");
                 close(s);
             } else if (fcntl(s, F_SETFL, 0) == -1) {
                 /* Switch off non-blocking mode, in case it is inherited. */
-                print_log(LOG_ERR, "listeners_post_select: fcntl(F_SETFL): %m");
+                log_print(LOG_ERR, "listeners_post_select: fcntl(F_SETFL): %m");
                 close(s);
             } else {
                 connection *J;
@@ -157,15 +154,15 @@ void listeners_post_select(fd_set *readfds, fd_set *writefds, fd_set *exceptfds)
                     xwrite(s, m, strlen(m));
                     shutdown(s, 2);
                     close(s);
-                    print_log(LOG_INFO, _("listeners_post_select: rejected connection from %s owing to high load"), inet_ntoa(sin.sin_addr));
+                    log_print(LOG_INFO, _("listeners_post_select: rejected connection from %s owing to high load"), inet_ntoa(sin.sin_addr));
                 } else {
                     /* Find a free connection slot. */
                     *J = connection_new(s, &sin, L->domain);
                     if (*J)
-                        print_log(LOG_INFO, _("listeners_post_select: client %s: connected"), (*J)->idstr);
+                        log_print(LOG_INFO, _("listeners_post_select: client %s: connected"), (*J)->idstr);
                     else
                         /* This could be really bad, but all we can do is log the failure. */
-                        print_log(LOG_ERR, _("listeners_post_select: unable to set up connection from %s: %m"), inet_ntoa(sin.sin_addr));
+                        log_print(LOG_ERR, _("listeners_post_select: unable to set up connection from %s: %m"), inet_ntoa(sin.sin_addr));
                 }
             }
         }
@@ -173,8 +170,7 @@ void listeners_post_select(fd_set *readfds, fd_set *writefds, fd_set *exceptfds)
 }
 
 /* connections_pre_select:
- * Called before the main select(2) so connections can be polled.
- */
+ * Called before the main select(2) so connections can be polled. */
 void connections_pre_select(int *n, fd_set *readfds, fd_set *writefds, fd_set *exceptfds) {
     connection *J;
     for (J = connections; J < connections + max_connections; ++J)
@@ -186,12 +182,8 @@ void connections_pre_select(int *n, fd_set *readfds, fd_set *writefds, fd_set *e
 }
 
 /* fork_child:
- * Handle forking a child to handle an individual connection c after authentication.
- */
-#ifdef AUTH_OTHER
-extern volatile int authchild_wr, authchild_rd; /* in auth_other.c */
-#endif /* AUTH_OTHER */
-
+ * Handle forking a child to handle an individual connection c after
+ * authentication. */
 void fork_child(connection c) {
     connection *J;
     item *t;
@@ -211,8 +203,7 @@ void fork_child(connection c) {
     switch((ch = fork())) {
         case 0:
             /* Child. Dispose of listeners and connections other than this
-             * one.
-             */
+             * one. */
             vector_iterate(listeners, t) listener_delete((listener)t->v);
             vector_delete(listeners);
             listeners = NULL;
@@ -225,28 +216,26 @@ void fork_child(connection c) {
                     *J = NULL;
                 }
             
-#ifdef AUTH_OTHER
-            /* Need to close the pipe to the authentication child. */
-            if (authchild_rd != -1) close(authchild_rd);
-            if (authchild_wr != -1) close(authchild_wr);
-#endif /* AUTH_OTHER */
+            /* Do any post-fork cleanup defined by authenticators. */
+            authswitch_postfork();
 
             /* We never access mailspools as root. */
             if (c->a->uid == 0) {
-                print_log(LOG_ERR, _("fork_child: client %s: authentication context has UID of 0"), c->idstr);
+                log_print(LOG_ERR, _("fork_child: client %s: authentication context has UID of 0"), c->idstr);
                 connection_sendresponse(c, 0, _("Everything's really bad"));
                 connection_delete(c);
                 _exit(0);
             }
 
-            /* Set our gid and uid to that appropriate for the mailspool, as decided by the auth switch. */
+            /* Set our gid and uid to that appropriate for the mailspool, as
+             * decided by the auth switch. */
             if (setgid(c->a->gid) == -1) {
-                print_log(LOG_ERR, "fork_child: setgid(%d): %m", c->a->gid);
+                log_print(LOG_ERR, "fork_child: setgid(%d): %m", c->a->gid);
                 connection_sendresponse(c, 0, _("Something bad happened, and I just can't go on. Sorry."));
                 connection_delete(c);
                 _exit(0);
             } else if (setuid(c->a->uid) == -1) {
-                print_log(LOG_ERR, "fork_child: setuid(%d): %m", c->a->uid);
+                log_print(LOG_ERR, "fork_child: setuid(%d): %m", c->a->uid);
                 connection_sendresponse(c, 0, _("Something bad happened, and I just can't go on. Sorry."));
                 connection_delete(c);
                 _exit(0);
@@ -279,16 +268,11 @@ void fork_child(connection c) {
                 _exit(0);
             }
 
-            /* Began session; log something useful in case of POP-before-SMTP
-             * relaying.
-             */
-            print_log(LOG_INFO, _("fork_child: %s: successfully authenticated with %s"), c->idstr, c->a->auth);
-
             break;
 
         case -1:
             /* Error. */
-            print_log(LOG_ERR, "fork_child: fork: %m");
+            log_print(LOG_ERR, "fork_child: fork: %m");
             connection_sendresponse(c, 0, _("Everything was fine until now, but suddenly I realise I just can't go on. Sorry."));
             remove_connection(c);
             connection_delete(c);
@@ -297,14 +281,21 @@ void fork_child(connection c) {
         default:
             /* Parent. Dispose of our copy of this connection. */
             post_fork = 0;  /* Now SIGHUP will work again. */
-            
+ 
+            /* Began session. We log a message in a known format, and call
+             * into the authentication drivers in case they want to do
+             * something with the information for POP-before-SMTP relaying. */
+            log_print(LOG_INFO, _("fork_child: %s: successfully authenticated with %s"), c->idstr, c->a->auth);
+            authswitch_onlogin(c->a, c->remote_ip, c->local_ip);
+           
+            /* Dispose of our copy of the connection. */
             close(c->s);
             c->s = -1; /* Don't shutdown the socket */
             remove_connection(c);
             connection_delete(c);
             c = NULL;
             
-            print_log(LOG_INFO, "fork_child: new child is PID %d", (int)ch);
+            log_print(LOG_INFO, "fork_child: new child is PID %d", (int)ch);
 
             ++num_running_children;
     }
@@ -316,8 +307,7 @@ void fork_child(connection c) {
 }
 
 /* connections_post_select:
- * Called after the main select(2) to do stuff with connections.
- */
+ * Called after the main select(2) to do stuff with connections. */
 void connections_post_select(fd_set *readfds, fd_set *writefds, fd_set *exceptfds) {
     connection *I;
     
@@ -330,13 +320,13 @@ void connections_post_select(fd_set *readfds, fd_set *writefds, fd_set *exceptfd
                 n = connection_read(c);
                 if (n == 0) {
                     /* Peer closed the connection */
-                    print_log(LOG_INFO, _("connections_post_select: connection_read: client %s: closed connection"), c->idstr);
+                    log_print(LOG_INFO, _("connections_post_select: connection_read: client %s: closed connection"), c->idstr);
                     connection_delete(c);
                     *I = NULL;
                     if (post_fork) _exit(0);
                 } else if (n < 0 && errno != EINTR) {
                     /* Some sort of error occurred, and we should close the connection. */
-                    print_log(LOG_ERR, _("connections_post_select: connection_read: client %s: disconnected: %m"), c->idstr);
+                    log_print(LOG_ERR, _("connections_post_select: connection_read: client %s: disconnected: %m"), c->idstr);
                     connection_delete(c);
                     *I = NULL;
                     if (post_fork) _exit(0);
@@ -348,7 +338,7 @@ void connections_post_select(fd_set *readfds, fd_set *writefds, fd_set *exceptfd
                         pop3command_delete(p);
                         switch (act) {
                             case close_connection:
-                                print_log(LOG_INFO, _("connections_post_select: client %s: disconnected"), c->idstr);
+                                log_print(LOG_INFO, _("connections_post_select: client %s: disconnected; %d/%d bytes read/written"), c->idstr, c->nrd, c->nwr);
                                 remove_connection(c);
                                 connection_delete(c);
                                 *I = c = NULL;
@@ -358,7 +348,7 @@ void connections_post_select(fd_set *readfds, fd_set *writefds, fd_set *exceptfd
                             case fork_and_setuid:
                                 if (num_running_children >= max_running_children) {
                                     connection_sendresponse(c, 0, _("Sorry, I'm too busy right now"));
-                                    print_log(LOG_INFO, _("connections_post_select: client %s: rejected login owing to high load"), c->idstr);
+                                    log_print(LOG_INFO, _("connections_post_select: client %s: rejected login owing to high load"), c->idstr);
                                     connection_delete(c);
                                     *I = c = NULL;
                                     break;
@@ -378,7 +368,7 @@ void connections_post_select(fd_set *readfds, fd_set *writefds, fd_set *exceptfd
 #else
                 connection_sendresponse(c, 0, _("Client has been idle for too long."));
 #endif
-                print_log(LOG_INFO, "net_loop: timed out client %s", c->idstr);
+                log_print(LOG_INFO, "net_loop: timed out client %s", c->idstr);
                 connection_delete(c);
                 *I = NULL;
                 if (post_fork) _exit(0);
@@ -389,8 +379,7 @@ void connections_post_select(fd_set *readfds, fd_set *writefds, fd_set *exceptfd
 
 /* net_loop:
  * Accept connections and put them into an appropriate state, calling
- * setuid() and fork() when appropriate.
- */
+ * setuid() and fork() when appropriate. */
 volatile int foad = 0, restart = 0; /* Flags used to indicate that we should exit or should re-exec. */
 
 #ifdef AUTH_OTHER
@@ -398,7 +387,7 @@ extern pid_t authchild_died;
 extern int authchild_status;
 #endif /* AUTH_OTHER */
 
-void net_loop() {
+void net_loop(void) {
     connection *J;
 #ifdef AUTH_OTHER
     sigset_t chmask;
@@ -410,7 +399,7 @@ void net_loop() {
     max_connections = 2 * max_running_children;
     connections = (connection*)xcalloc(max_connections, sizeof(connection*));
 
-    print_log(LOG_INFO, _("net_loop: tpop3d version %s successfully started"), TPOP3D_VERSION);
+    log_print(LOG_INFO, _("net_loop: tpop3d version %s successfully started"), TPOP3D_VERSION);
     
     /* Main select() loop */
     while (!foad) {
@@ -426,7 +415,7 @@ void net_loop() {
 
         e = select(n + 1, &readfds, NULL, NULL, &tv);
         if (e == -1 && errno != EINTR) {
-            print_log(LOG_WARNING, "net_loop: select: %m");
+            log_print(LOG_WARNING, "net_loop: select: %m");
         } else if (e >= 0) {
             /* Check for new incoming connections */
             if (!post_fork) listeners_post_select(&readfds, NULL, NULL);
@@ -438,11 +427,10 @@ void net_loop() {
 #ifdef AUTH_OTHER
         /* It may be that the authentication child died; log the message here
          * to avoid doing something we shouldn't in the signal handler. We
-         * block SIGCHLD while doing this.
-         */
+         * block SIGCHLD while doing this. */
         sigprocmask(SIG_BLOCK, &chmask, NULL);
         if (authchild_died) {
-            print_log(LOG_WARNING, _("net_loop: authentication child %d terminated with status %d"), (int)authchild_died, authchild_status);
+            log_print(LOG_WARNING, _("net_loop: authentication child %d terminated with status %d"), (int)authchild_died, authchild_status);
             authchild_died = 0;
         }
         sigprocmask(SIG_UNBLOCK, &chmask, NULL);
@@ -450,11 +438,11 @@ void net_loop() {
     }
 
     /* Termination request received; we should close all connections in an
-     * orderly fashion.
-     */
-    if (restart) print_log(LOG_INFO, _("net_loop: restarting on signal %d"),
-            foad);
-    else print_log(LOG_INFO, _("net_loop: terminating on signal %d"), foad);
+     * orderly fashion. */
+    if (restart)
+        log_print(LOG_INFO, _("net_loop: restarting on signal %d"), foad);
+    else
+        log_print(LOG_INFO, _("net_loop: terminating on signal %d"), foad);
 
     if (connections) {
         for (J = connections; J < connections + max_connections; ++J)
@@ -493,8 +481,8 @@ void usage(FILE *fp) {
 #endif
     
     fprintf(fp, _(
-"tpop3d, copyright (c) 2000-2001 Chris Lightfoot <chris@ex-parrot.com>;\n"
-"portions copyright (c) 2001 Mark Longair, Paul Makepeace.\n"
+"tpop3d, copyright (c) 2000-2 Chris Lightfoot <chris@ex-parrot.com>;\n"
+"portions copyright (c) 2001-2 Mark Longair, Paul Makepeace, Sebastien Thomas.\n"
 "home page: http://www.ex-parrot.com/~chris/tpop3d/\n"
 "\n"
 "This program is free software; you can redistribute it and/or modify\n"
@@ -506,8 +494,7 @@ void usage(FILE *fp) {
 }
 
 /* main:
- * Read config file, set up authentication and proceed to main loop.
- */
+ * Read config file, set up authentication and proceed to main loop. */
 char optstring[] = "+hdvf:p:";
 
 #if defined(MBOX_BSD) && defined(MBOX_BSD_SAVE_INDICES)
@@ -515,9 +502,8 @@ extern int mailspool_save_indices;  /* in mailspool.c */
 #endif
 
 int main(int argc, char **argv, char **envp) {
-    item *I;
     int nodaemon = 0;
-    char *configfile = "/etc/tpop3d.conf";
+    char *configfile = "/etc/tpop3d.conf", *s;
     int na, c;
 
 #ifdef MTRACE_DEBUGGING
@@ -564,27 +550,37 @@ int main(int argc, char **argv, char **envp) {
     config = read_config_file(configfile);
     if (!config) return 1;
 
+    /* The config file may specify that we aren't to run in daemon mode. */
+    if (config_get_bool("no-detach"))
+        nodaemon = 1;
+
+    /* ... or that we are to log to standard error. */
+    if (config_get_bool("log-stderr")) {
+        if (nodaemon)
+            log_stderr = 1;
+        else
+            fprintf(stderr, _("tpop3d: will not log to standard error when running detached"));
+    }
+
     /* Detach from controlling tty etc. */
     if (!nodaemon) daemon(0, 0);
 
     /* Start logging. */
-    openlog("tpop3d", LOG_PID | LOG_NDELAY, LOG_MAIL);
+    log_init();
 
     /* Perhaps we have been asked to save metadata caches for BSD mailspools? */
 #if defined(MBOX_BSD) && defined(MBOX_BSD_SAVE_INDICES)
-    I = stringmap_find(config, "mailspool-index");
-    if (I) {
+    if (config_get_string("mailspool-index")) {
         mailspool_save_indices = 1;
-        print_log(LOG_INFO, _("experimental BSD mailbox metadata cache enabled"));
+        log_print(LOG_INFO, _("experimental BSD mailbox metadata cache enabled"));
     }
 #endif
 
     /* We may have been compiled with TCP wrappers support. */
 #ifdef USE_TCP_WRAPPERS
-    I = stringmap_find(config, "tcp-wrappers-name");
-    if (I)
-        tcpwrappersname = I->v;
-    print_log(LOG_INFO, _("TCP Wrappers support enabled, using daemon name `%s'"), tcpwrappersname);
+    if (!(tcpwrappersname = config_get_string("tcp-wrappers-name")))
+        tcpwrappersname = "tpop3d";
+    log_print(LOG_INFO, _("TCP Wrappers support enabled, using daemon name `%s'"), tcpwrappersname);
 #endif
     
     /* Try to write PID file. */
@@ -599,40 +595,41 @@ retry_pid_file:
                 switch (read_pid_file(pidfile, &pid)) {
                     case pid_file_success:
                         if (kill(pid, 0)) {
-                            print_log(LOG_ERR, _("%s: stale PID file; removing it"), pidfile);
+                            log_print(LOG_ERR, _("%s: stale PID file; removing it"), pidfile);
                             if (unlink(pidfile) == -1) {
-                                print_log(LOG_ERR, _("%s: stale PID file: unlink: %m"), pidfile);
+                                log_print(LOG_ERR, _("%s: stale PID file: unlink: %m"), pidfile);
                                 return 1;
                             } else goto retry_pid_file; /* harmful? */
                             
                         } else {
-                            print_log(LOG_ERR, _("%s: tpop3d already running, with process ID %d; exiting"), (int)pid);
+                            log_print(LOG_ERR, _("%s: tpop3d already running, with process ID %d; exiting"), (int)pid);
                             return 1;
                         }
                         break;
                         
                     default:
-                        print_log(LOG_ERR, _("%s: PID file seems to be invalid; exiting."), pidfile);
+                        log_print(LOG_ERR, _("%s: PID file seems to be invalid; exiting."), pidfile);
                         return 1;
                 }
                 break;
             }
 
             case pid_file_error:
-                print_log(LOG_ERR, _("%s: %m: couldn't write PID file; exiting."), pidfile);
+                log_print(LOG_ERR, _("%s: %m: couldn't write PID file; exiting."), pidfile);
                 return 1;
         }
     }
 
     /* Identify addresses on which to listen.
-     * The syntax for these is <addr>[:port][(domain)].
-     */
-    I = stringmap_find(config, "listen-address");
+     * The syntax for these is <addr>[:port][(domain)]. */
+    s = config_get_string("listen-address");
     listeners = vector_new();
-    if (I) {
-        tokens t = tokens_new(I->v, " \t");
+    if (s) {
+        tokens t;
         char **J;
 
+        t = tokens_new(s, " \t");
+
         for (J = t->toks; J < t->toks + t->num; ++J) {
             struct sockaddr_in sin = {0};
             listener L;
@@ -644,7 +641,7 @@ retry_pid_file:
             r = strchr(s, '(');
             if (r) {
                 if (*(s + strlen(s) - 1) != ')') {
-                    print_log(LOG_ERR, _("%s: syntax for listen address `%s' is incorrect"), configfile, s);
+                    log_print(LOG_ERR, _("%s: syntax for listen address `%s' is incorrect"), configfile, s);
                     continue;
                 }
 
@@ -662,7 +659,7 @@ retry_pid_file:
                     struct servent *se;
                     se = getservbyname(r, "tcp");
                     if (!se) {
-                        print_log(LOG_ERR, _("%s: specified listen address `%s' has invalid port `%s'"), configfile, s, r);
+                        log_print(LOG_ERR, _("%s: specified listen address `%s' has invalid port `%s'"), configfile, s, r);
                         continue;
                     } else sin.sin_port = se->s_port;
                 } else sin.sin_port = htons(sin.sin_port);
@@ -673,7 +670,7 @@ retry_pid_file:
                 struct hostent *he;
                 he = gethostbyname(s);
                 if (!he) {
-                    print_log(LOG_ERR, _("%s: gethostbyname: specified listen address `%s' is invalid"), configfile, s);
+                    log_print(LOG_ERR, _("%s: gethostbyname: specified listen address `%s' is invalid"), configfile, s);
                     continue;
                 } else memcpy(&(sin.sin_addr), he->h_addr, sizeof(struct in_addr));
             }
@@ -681,7 +678,7 @@ retry_pid_file:
             L = listener_new(&sin, domain);
             if (L) {
                 vector_push_back(listeners, item_ptr(L));
-                print_log(LOG_INFO, _("listening on address %s, port %d, domain %s"), inet_ntoa(L->sin.sin_addr), htons(L->sin.sin_port), (L->domain ? L->domain : _("(none)")));
+                log_print(LOG_INFO, _("listening on address %s, port %d, domain %s"), inet_ntoa(L->sin.sin_addr), htons(L->sin.sin_port), (L->domain ? L->domain : _("(none)")));
             }
         }
 
@@ -689,19 +686,19 @@ retry_pid_file:
     }
 
     if (listeners->n_used == 0) {
-        print_log(LOG_ERR, _("%s: no listen addresses obtained; exiting"), configfile);
+        log_print(LOG_ERR, _("%s: no listen addresses obtained; exiting"), configfile);
         EXIT_REMOVING_PIDFILE(1);
     }
 
     /* Find out the maximum number of children we may spawn at once. */
     switch(config_get_int("max-children", &max_running_children)) {
         case -1:
-            print_log(LOG_ERR, _("%s: value given for max-children does not make sense; exiting"), configfile);
+            log_print(LOG_ERR, _("%s: value given for max-children does not make sense; exiting"), configfile);
             EXIT_REMOVING_PIDFILE(1);
 
         case 1:
             if (max_running_children < 1) {
-                print_log(LOG_ERR, _("%s: value for max-children must be 1 or greater; exiting"), configfile);
+                log_print(LOG_ERR, _("%s: value for max-children must be 1 or greater; exiting"), configfile);
                 EXIT_REMOVING_PIDFILE(1);
             }
             break;
@@ -710,19 +707,28 @@ retry_pid_file:
             max_running_children = 16;
     }
 
-    /* Should we automatically append domain names and retry authentication? */
-    I = stringmap_find(config, "append-domain");
-    if (I && (!strcmp(I->v, "yes") || !strcmp(I->v, "true"))) append_domain = 1;
+    /* Should we automatically append or strip domain names and retry authentication? */
+    if (config_get_bool("append-domain"))
+        append_domain = 1;
+    if (config_get_bool("strip-domain"))
+        strip_domain = 1;
 
-    /* Find out how long we wait before timing out... */
+    if (append_domain && strip_domain)
+        log_print(LOG_WARNING, _("%s: specifying append-domain and strip-domain does not make much sense"));
+
+    /* Should we disconnect any client which sends a USER command? */
+    if (config_get_bool("apop-only"))
+        apop_only = 1;
+
+    /* Find out how long we wait before timing out.... */
     switch (config_get_int("timeout-seconds", &timeout_seconds)) {
         case -1:
-            print_log(LOG_ERR, _("%s: value given for timeout-seconds does not make sense; exiting"), configfile);
+            log_print(LOG_ERR, _("%s: value given for timeout-seconds does not make sense; exiting"), configfile);
             EXIT_REMOVING_PIDFILE(1);
 
         case 1:
             if (timeout_seconds < 1) {
-                print_log(LOG_ERR, _("%s: value for timeout-seconds must be 1 or greater; exiting"), configfile);
+                log_print(LOG_ERR, _("%s: value for timeout-seconds must be 1 or greater; exiting"), configfile);
                 EXIT_REMOVING_PIDFILE(1);
             }
             break;
@@ -733,18 +739,21 @@ retry_pid_file:
 
     set_signals();
 
-    /* Start the authentication drivers */
+    /* Start the authentication drivers. */
     na = authswitch_init();
     if (!na) {
-        print_log(LOG_ERR, _("no authentication drivers were loaded; aborting."));
-        print_log(LOG_ERR, _("you may wish to check your config file %s"), configfile);
+        log_print(LOG_ERR, _("no authentication drivers were loaded; aborting."));
+        log_print(LOG_ERR, _("you may wish to check your config file %s"), configfile);
         EXIT_REMOVING_PIDFILE(1);
-    } else print_log(LOG_INFO, _("%d authentication drivers successfully loaded"), na);
+    } else log_print(LOG_INFO, _("%d authentication drivers successfully loaded"), na);
    
     net_loop();
 
-    authswitch_close();
+    if (!post_fork)
+        authswitch_close();
+
     if (listeners) {
+        item *I;
         vector_iterate(listeners, I) listener_delete((listener)I->v);
         vector_delete(listeners);
     }
@@ -754,7 +763,7 @@ retry_pid_file:
     /* We may have got here because we're supposed to terminate and restart. */
     if (restart) {
         execve(argv[0], argv, envp);
-        print_log(LOG_ERR, "%s: %m", argv[0]);
+        log_print(LOG_ERR, "%s: %m", argv[0]);
     }
     
     EXIT_REMOVING_PIDFILE(0);
