@@ -6,7 +6,7 @@
  *
  */
 
-static const char rcsid[] = "$Id: pop3.c,v 1.40 2002/03/18 23:47:33 chris Exp $";
+static const char rcsid[] = "$Id: pop3.c,v 1.42 2002/06/25 20:28:00 chris Exp $";
 
 #ifdef HAVE_CONFIG_H
 #include "configuration.h"
@@ -33,7 +33,8 @@ extern int verbose;
  * Makes a command do something on a connection, returning a code indicating
  * what the caller should do. */
 int append_domain;  /* Do we automatically try user@domain if user alone fails to authenticate? */
-int strip_domain; /* Automatically try user if user@domain fails? */
+int strip_domain;   /* Automatically try user if user@domain fails? */
+int apop_only;      /* Disconnect any client which says USER. */
 
 enum connection_action connection_do(connection c, const pop3command p) {
     /* This breaks the RFC, but is sensible. */
@@ -43,7 +44,10 @@ enum connection_action connection_do(connection c, const pop3command p) {
         /* Authorisation state: gather username and password. */
         switch (p->cmd) {
         case USER:
-            if (p->toks->num != 2) {
+            if (apop_only) {
+                connection_sendresponse(c, 0, _("Sorry, you must use APOP."));
+                return close_connection;
+            } else if (p->toks->num != 2) {
                 connection_sendresponse(c, 0, _("No, that's not right."));
                 return do_nothing;
             } else if (c->user) {
@@ -125,7 +129,7 @@ enum connection_action connection_do(connection c, const pop3command p) {
                     return do_nothing;
                 }
 
-                c->a = authcontext_new_apop(name, NULL, c->domain, c->timestamp, digest, inet_ntoa(c->sin.sin_addr));
+                c->a = authcontext_new_apop(name, NULL, c->domain, c->timestamp, digest, c->remote_ip, c->local_ip);
  
                 /* Maybe retry authentication with an added or removed domain
                  * name. */
@@ -135,13 +139,13 @@ enum connection_action connection_do(connection c, const pop3command p) {
                     n = strcspn(name, DOMAIN_SEPARATORS);
                     if (append_domain && c->domain && n == len)
                         /* OK, if we have a domain name, try appending that. */
-                        c->a = authcontext_new_apop(name, name, c->domain, c->timestamp, digest, inet_ntoa(c->sin.sin_addr));
+                        c->a = authcontext_new_apop(name, name, c->domain, c->timestamp, digest, c->remote_ip, c->local_ip);
                     else if (strip_domain && n != len) {
                         /* Try stripping off the supplied domain name. */
                         char *u;
                         u = xstrdup(name);
                         u[n] = 0;
-                        c->a = authcontext_new_apop(u, NULL, NULL, c->timestamp, digest, inet_ntoa(c->sin.sin_addr));
+                        c->a = authcontext_new_apop(u, NULL, NULL, c->timestamp, digest, c->remote_ip, c->local_ip);
                         xfree(u);
                     }
                 }
@@ -200,7 +204,7 @@ enum connection_action connection_do(connection c, const pop3command p) {
 
         /* Do we now have enough information to authenticate using USER/PASS? */
         if (!c->a && c->user && c->pass) {
-            c->a = authcontext_new_user_pass(c->user, NULL, c->domain, c->pass, inet_ntoa(c->sin.sin_addr));
+            c->a = authcontext_new_user_pass(c->user, NULL, c->domain, c->pass, c->remote_ip, c->local_ip);
             
             /* Maybe retry authentication with an added or removed domain name. */
             if (!c->a && (append_domain || strip_domain)) {
@@ -209,13 +213,13 @@ enum connection_action connection_do(connection c, const pop3command p) {
                 n = strcspn(c->user, DOMAIN_SEPARATORS);
                 if (append_domain && c->domain && n == len)
                     /* OK, if we have a domain name, try appending that. */
-                    c->a = authcontext_new_user_pass(c->user, c->user, c->domain, c->pass, inet_ntoa(c->sin.sin_addr));
+                    c->a = authcontext_new_user_pass(c->user, c->user, c->domain, c->pass, c->remote_ip, c->local_ip);
                 else if (strip_domain && n != len) {
                     /* Try stripping off the supplied domain name. */
                     char *u;
                     u = xstrdup(c->user);
                     u[n] = 0;
-                    c->a = authcontext_new_user_pass(u, NULL, NULL, c->pass, inet_ntoa(c->sin.sin_addr));
+                    c->a = authcontext_new_user_pass(u, NULL, NULL, c->pass, c->remote_ip, c->local_ip);
                     xfree(u);
                 }
             }
@@ -410,14 +414,18 @@ enum connection_action connection_do(connection c, const pop3command p) {
                 if (curmsg->deleted)
                     connection_sendresponse(c, 0, _("That message is no more."));
                 else {
+                    int n;
+
                     if (verbose)
                         log_print(LOG_DEBUG, _("connection_do: client %s: sending message %d (%d bytes)"),
                                     c->idstr, msg_num + 1, (int)curmsg->msglength);
                     connection_sendresponse(c, 1, _("Message follows:"));
-                    if (!(curmbox)->send_message(curmbox, c->s, msg_num, -1)) {
+                    if ((n = (curmbox)->send_message(curmbox, c->s, msg_num, -1)) == -1) {
                         connection_sendresponse(c, 0, _("Oops"));
                         return close_connection;
                     }
+                    c->nwr += n; /* Record bytes sent. */
+                    
                     /* That might have taken a long time. */
                     c->idlesince = time(NULL);
                     if (verbose)
@@ -430,6 +438,8 @@ enum connection_action connection_do(connection c, const pop3command p) {
             }
 
         case TOP: {
+                int n;
+
                 if (!have_msg_num) {
                     connection_sendresponse(c, 0, _("What do you want to see?"));
                     break;
@@ -446,10 +456,12 @@ enum connection_action connection_do(connection c, const pop3command p) {
                                 c->idstr, arg2, msg_num + 1, (int)curmsg->msglength);
                 connection_sendresponse(c, 1, _("Message follows:"));
 
-                if (!(curmbox)->send_message(curmbox, c->s, msg_num, arg2)) {
+                if ((n = (curmbox)->send_message(curmbox, c->s, msg_num, arg2)) == -1) {
                     connection_sendresponse(c, 0, _("Oops."));
                     return close_connection;
                 }
+                c->nwr += n; /* Record bytes sent. */
+
                 /* That might have taken a long time. */
                 c->idlesince = time(NULL);
                 if (verbose)
@@ -507,8 +519,7 @@ enum connection_action connection_do(connection c, const pop3command p) {
 
 /* connection_start_transaction:
  * Set up the connection into the "transaction" state. Returns 1 on success or
- * 0 on failure.
- */
+ * 0 on failure. */
 int connection_start_transaction(connection c) {
     if (!c) return 0;
     
