@@ -4,7 +4,7 @@
  *
  * Copyright (c) 2001 Chris Lightfoot. All rights reserved.
  *
- * $Id: util.h,v 1.17 2001/10/14 01:13:38 chris Exp $
+ * $Id: util.h,v 1.22 2002/03/18 23:47:34 chris Exp $
  *
  */
 
@@ -28,6 +28,10 @@
 #   define PAGESIZE        getpagesize()
 #endif
 
+#ifndef DOMAIN_SEPARATORS
+#   define DOMAIN_SEPARATORS    "@%!:"
+#endif
+
 #if 0
 /* Primitive memory-leak debugging. */
 char *mystrdup(char *f, int l, const char *s);
@@ -46,14 +50,16 @@ char *xstrncat(char *pfx, const char *sfx, const size_t n);
 
 /* Function for substituting $(...) in strings. */
 struct sverr {
+    enum {sv_ok = 0, sv_syntax, sv_unknown, sv_range, sv_nullvalue } code;
     char *msg;
     off_t offset;
 };
 
 char *substitute_variables(const char *spec, struct sverr *err, const int nvars, ...);
 
-/* Replacement logging function. */
-void print_log(int priority, const char *fmt, ...);
+/* Replacement logging functions. */
+void log_init(void);
+void log_print(int priority, const char *fmt, ...);
 
 /* Restarting write(2). */
 ssize_t xwrite(int fd, const void *buf, size_t count);
@@ -85,10 +91,26 @@ char *hex_digest(const unsigned char *u);
 int unhex_digest(const char *from, unsigned char *to);
 
 /* Memory allocation wrappers. */
+#ifndef MTRACE_DEBUGGING
 void *xmalloc(size_t n);
 void *xcalloc(size_t n, size_t m);
 void *xrealloc(void *w, size_t n);
 void xfree(void *v);
+char *xstrdup(const char *s);
+#else
+/* Malloc wrappers are incompatible with mtrace debugging because the log file
+ * produced by mtrace records only the innermost calling stack frame, and
+ * therefore reports all leaks as occurring in xmalloc, xcalloc etc. */
+#   define xmalloc  malloc
+#   define xcalloc  calloc
+#   define xrealloc realloc
+#   define xfree    free
+#   define xstrdup  strdup
+#endif /* !MTRACE_DEBUGGING */
+
+/* MD5 digests. */
+void md5_digest(const void *v, const size_t n, unsigned char *md5);
+char *md5_digest_str(const void *v, const size_t n, const int base64);
 
 /* Optional internationalisation support. */
 #ifdef WITH_I18N
